from reliabilly.components.controllers.data_controller import DataController
from reliabilly.settings import Settings


class {{service|camelcase}}Controller(DataController):
    name = Settings.SERVICE_NAME

