# {{service|capitalize}} Service

The {{service}} service is intended to <UPDATE AND DEFINE PURPOSE HERE>


## {{service|capitalize}} is available at /{{service}}/
- Create a new {{service}} (POST /{{service}}/)
- Delete a {{service}} (DELETE /{{service}}/<id>)
- Update a {{service}} (PUT /{{service}}/<id>)
- Get a {{service}} by Id (GET /{{service}}/<ID>)
- Get a list of {{service}} by page (GET /{{service}}/?limit=25&offset=0) 
- Filter the list of {{service}} (GET /{{service}}/?name=myName)
