from nameko.runners import ServiceRunner
from .controllers.{{service}}_controller import {{service|camelcase}}Controller
from .settings import Constants


RUNNER = ServiceRunner(config=Constants.NAMEKO_CONFIG)
RUNNER.add_service({{service|camelcase}}Controller)

